@@ -6,25 +6,25 @@
 #
 from __future__ import annotations
 
-from io import BufferedWriter, BytesIO
 import itertools
 import jwt
 import logging
+import os
 import pathlib
 import requests
-import shlex
-import shutil
-import subprocess
 import sys
+import tempfile
 import time
+
+from contextlib import nullcontext
+from io import BufferedWriter, BytesIO
+from requests.compat import json  # type: ignore
 from typing import Any, ContextManager, Optional, overload, Sequence, Tuple, List
-import urllib.request
+from packaging.version import Version, parse
 
 import firecrest.FirecrestException as fe
 import firecrest.types as t
-
-from contextlib import nullcontext
-from requests.compat import json  # type: ignore
+from firecrest.ExternalStorage import ExternalUpload, ExternalDownload
 
 if sys.version_info >= (3, 8):
     from typing import Literal
@@ -33,6 +33,7 @@ else:
 
 logger = logging.getLogger(__name__)
 
+
 # This function is temporarily here
 def handle_response(response):
     print("\nResponse status code:")
@@ -46,215 +47,6 @@ def handle_response(response):
         print("-")
 
 
-class ExternalStorage:
-    """External storage object.
-    """
-
-    _final_states: set[str]
-
-    def __init__(
-        self,
-        client: Firecrest,
-        task_id: str,
-        previous_responses: Optional[List[requests.Response]] = None,
-    ) -> None:
-        previous_responses = [] if previous_responses is None else previous_responses
-        self._client = client
-        self._task_id = task_id
-        self._in_progress = True
-        self._status: Optional[str] = None
-        self._data = None
-        self._object_storage_data = None
-        self._sleep_time = itertools.cycle([1, 5, 10])
-        self._responses = previous_responses
-
-    @property
-    def client(self) -> Firecrest:
-        """Returns the client that will be used to get information for the task.
-        """
-        return self._client
-
-    @property
-    def task_id(self) -> str:
-        """Returns the FirecREST task ID that is associated with this transfer.
-        """
-        return self._task_id
-
-    def _update(self) -> None:
-        if self._status not in self._final_states:
-            task = self._client._task_safe(self._task_id, self._responses)
-            self._status = task["status"]
-            self._data = task["data"]
-            logger.info(f"Task {self._task_id} has status {self._status}")
-            if not self._object_storage_data:
-                if self._status == "111":
-                    self._object_storage_data = task["data"]["msg"]
-                elif self._status == "117":
-                    self._object_storage_data = task["data"]
-
-    @property
-    def status(self) -> str:
-        """Returns status of the task that is associated with this transfer.
-
-        :calls: GET `/tasks/{taskid}`
-        """
-        self._update()
-        return self._status  # type: ignore
-
-    @property
-    def in_progress(self) -> bool:
-        """Returns `False` when the transfer has been completed (succesfully or with errors), otherwise `True`.
-
-        :calls: GET `/tasks/{taskid}`
-        """
-        self._update()
-        return self._status not in self._final_states
-
-    @property
-    def data(self) -> Optional[dict]:
-        """Returns the task information from the latest response.
-
-        :calls: GET `/tasks/{taskid}`
-        """
-        self._update()
-        return self._data
-
-    @property
-    def object_storage_data(self):
-        """Returns the necessary information for the external transfer.
-        The call is blocking and in cases of large file transfers it might take a long time.
-
-        :calls: GET `/tasks/{taskid}`
-        :rtype: dictionary or string
-        """
-        if not self._object_storage_data:
-            self._update()
-
-        while not self._object_storage_data:
-            t = next(self._sleep_time)
-            logger.info(f"Sleeping for {t} sec")
-            time.sleep(t)
-            self._update()
-
-        return self._object_storage_data
-
-
-class ExternalUpload(ExternalStorage):
-    """
-    This class handles the external upload from a file.
-
-    Tracks the progress of the upload through the status of the associated task.
-    Final states: *114* and *115*.
-
-    +--------+--------------------------------------------------------------------+
-    | Status | Description                                                        |
-    +========+====================================================================+
-    | 110    | Waiting for Form URL from Object Storage to be retrieved           |
-    +--------+--------------------------------------------------------------------+
-    | 111    | Form URL from Object Storage received                              |
-    +--------+--------------------------------------------------------------------+
-    | 112    | Object Storage confirms that upload to Object Storage has finished |
-    +--------+--------------------------------------------------------------------+
-    | 113    | Download from Object Storage to server has started                 |
-    +--------+--------------------------------------------------------------------+
-    | 114    | Download from Object Storage to server has finished                |
-    +--------+--------------------------------------------------------------------+
-    | 115    | Download from Object Storage error                                 |
-    +--------+--------------------------------------------------------------------+
-
-    :param client: FirecREST client associated with the transfer
-    :param task_id: FirecrREST task associated with the transfer
-    """
-
-    def __init__(
-        self,
-        client: Firecrest,
-        task_id: str,
-        previous_responses: Optional[List[requests.Response]] = None,
-    ) -> None:
-        previous_responses = [] if previous_responses is None else previous_responses
-        super().__init__(client, task_id, previous_responses)
-        self._final_states = {"114", "115"}
-        logger.info(f"Creating ExternalUpload object for task {task_id}")
-
-    def finish_upload(self) -> None:
-        """Finish the upload process.
-        This call will upload the file to the staging area.
-        Check with the method `status` or `in_progress` to see the status of the transfer.
-        The transfer from the staging area to the systems's filesystem can take several seconds to start to start.
-        """
-        c = self.object_storage_data["command"]  # typer: ignore
-        # LOCAL FIX FOR MAC
-        # c = c.replace("192.168.220.19", "localhost")
-        logger.info(f"Uploading the file to the staging area with the command: {c}")
-        command = subprocess.run(
-            shlex.split(c), stdout=subprocess.PIPE, stderr=subprocess.PIPE
-        )
-        if command.returncode != 0:
-            exc = Exception(
-                f"Failed to finish upload with error: {command.stderr.decode('utf-8')}"
-            )
-            logger.critical(exc)
-            raise exc
-
-
-class ExternalDownload(ExternalStorage):
-    """
-    This class handles the external download from a file.
-
-    Tracks the progress of the download through the status of the associated task.
-    Final states: *117* and *118*.
-
-    +--------+--------------------------------------------------------------------+
-    | Status | Description                                                        |
-    +========+====================================================================+
-    | 116    | Started upload from filesystem to Object Storage                   |
-    +--------+--------------------------------------------------------------------+
-    | 117    | Upload from filesystem to Object Storage has finished successfully |
-    +--------+--------------------------------------------------------------------+
-    | 118    | Upload from filesystem to Object Storage has finished with errors  |
-    +--------+--------------------------------------------------------------------+
-
-    :param client: FirecREST client associated with the transfer
-    :param task_id: FirecrREST task associated with the transfer
-    """
-
-    def __init__(
-        self,
-        client: Firecrest,
-        task_id: str,
-        previous_responses: Optional[List[requests.Response]] = None,
-    ) -> None:
-        previous_responses = [] if previous_responses is None else previous_responses
-        super().__init__(client, task_id, previous_responses)
-        self._final_states = {"117", "118"}
-        logger.info(f"Creating ExternalDownload object for task {task_id}")
-
-    def invalidate_object_storage_link(self) -> None:
-        """Invalidate the temporary URL for downloading.
-
-        :calls: POST `/storage/xfer-external/invalidate`
-        """
-        self._client._invalidate(self._task_id)
-
-    def finish_download(self, target_path: str | pathlib.Path | BufferedWriter) -> None:
-        """Finish the download process.
-
-        :param target_path: the local path to save the file
-        """
-        url = self.object_storage_data
-        logger.info(f"Downloading the file from {url} and saving to {target_path}")
-        # LOCAL FIX FOR MAC
-        # url = url.replace("192.168.220.19", "localhost")
-        context: ContextManager[BufferedWriter] = (
-            open(target_path, "wb")  # type: ignore
-            if isinstance(target_path, str) or isinstance(target_path, pathlib.Path)
-            else nullcontext(target_path)
-        )
-        with urllib.request.urlopen(url) as response, context as out_file:
-            shutil.copyfileobj(response, out_file)
-
-
 class Firecrest:
     """
     This is the basic class you instantiate to access the FirecREST API v1.
@@ -326,6 +118,14 @@ class Firecrest:
         #: Number of retries in case the rate limit is reached. When it is set to `None`, the
         #: client will keep trying until it gets a different status code than 429.
         self.num_retries_rate_limit: Optional[int] = None
+        self._api_version: Version = parse("1.13.1")
+        self._session = requests.Session()
+
+    def set_api_version(self, api_version: str) -> None:
+        """Set the version of the api of firecrest. By default it will be assumed that you are
+        using version 1.13.1 or compatible. The version is parsed by the `packaging` library.
+        """
+        self._api_version = parse(api_version)
 
     @_retry_requests  # type: ignore
     def _get_request(
@@ -337,7 +137,7 @@ class Firecrest:
             headers.update(additional_headers)
 
         logger.info(f"Making GET request to {endpoint}")
-        resp = requests.get(
+        resp = self._session.get(
             url=url,
             headers=headers,
             params=params,
@@ -356,7 +156,7 @@ class Firecrest:
             headers.update(additional_headers)
 
         logger.info(f"Making POST request to {endpoint}")
-        resp = requests.post(
+        resp = self._session.post(
             url=url,
             headers=headers,
             data=data,
@@ -376,7 +176,7 @@ class Firecrest:
             headers.update(additional_headers)
 
         logger.info(f"Making PUT request to {endpoint}")
-        resp = requests.put(
+        resp = self._session.put(
             url=url,
             headers=headers,
             data=data,
@@ -395,7 +195,7 @@ class Firecrest:
             headers.update(additional_headers)
 
         logger.info(f"Making DELETE request to {endpoint}")
-        resp = requests.delete(
+        resp = self._session.delete(
             url=url,
             headers=headers,
             data=data,
@@ -481,30 +281,22 @@ class Firecrest:
         responses: Optional[List[requests.Response]] = None,
     ) -> dict[str, t.Task]:
         """Return a dictionary of FirecREST tasks and their last update.
-        When `task_ids` is an empty list or contains more than one element the
-        `/tasks` endpoint will be called. Otherwise `/tasks/{taskid}`.
-        When the `/tasks` is called the method will not give an error for invalid IDs,
-        but `/tasks/{taskid}` will raise an exception.
+        The result will only contain entries for valid tasks and the rest will be ignored.
 
         :param task_ids: list of task IDs. When empty all tasks are returned.
         :param responses: list of responses that are associated with these tasks (only relevant for error)
-        :calls: GET `/tasks` or `/tasks/{taskid}`
+        :calls: GET `/tasks`
         """
         task_ids = [] if task_ids is None else task_ids
         responses = [] if responses is None else responses
-        endpoint = "/tasks/"
-        if len(task_ids) == 1:
-            endpoint += task_ids[0]
+        endpoint = "/tasks"
+        params = {}
+        if task_ids:
+            params = {"tasks": ",".join([str(j) for j in task_ids])}
 
-        resp = self._get_request(endpoint=endpoint)
+        resp = self._get_request(endpoint=endpoint, params=params)
         responses.append(resp)
-        taskinfo = self._json_response(responses, 200)
-        if len(task_ids) == 0:
-            return taskinfo["tasks"]
-        elif len(task_ids) == 1:
-            return {task_ids[0]: taskinfo["task"]}
-        else:
-            return {k: v for k, v in taskinfo["tasks"].items() if k in task_ids}
+        return self._json_response(responses, 200)["tasks"]
 
     def _task_safe(
         self, task_id: str, responses: Optional[List[requests.Response]] = None
@@ -627,8 +419,9 @@ class Firecrest:
         )
         return self._json_response([resp], 200)["output"]
 
-    def mkdir(self, machine: str, target_path: str, p: Optional[bool] = None) -> None:
+    def mkdir(self, machine: str, target_path: str, p: Optional[bool] = None) -> str:
         """Creates a new directory.
+        When successful, the method returns a string with the path of the newly created directory.
 
         :param machine: the machine name where the filesystem belongs to
         :param target_path: the absolute target path
@@ -645,9 +438,11 @@ class Firecrest:
             data=data,
         )
         self._json_response([resp], 201)
+        return target_path
 
-    def mv(self, machine: str, source_path: str, target_path: str) -> None:
+    def mv(self, machine: str, source_path: str, target_path: str) -> str:
         """Rename/move a file, directory, or symlink at the `source_path` to the `target_path` on `machine`'s filesystem.
+        When successful, the method returns a string with the new path of the file.
 
         :param machine: the machine name where the filesystem belongs to
         :param source_path: the absolute source path
@@ -660,6 +455,7 @@ class Firecrest:
             data={"targetPath": target_path, "sourcePath": source_path},
         )
         self._json_response([resp], 200)
+        return target_path
 
     def chmod(self, machine: str, target_path: str, mode: str) -> None:
         """Changes the file mod bits of a given file according to the specified mode.
@@ -709,8 +505,9 @@ class Firecrest:
         )
         self._json_response([resp], 200)
 
-    def copy(self, machine: str, source_path: str, target_path: str) -> None:
+    def copy(self, machine: str, source_path: str, target_path: str) -> str:
         """Copies file from `source_path` to `target_path`.
+        When successful, the method returns a string with the path of the newly created file.
 
         :param machine: the machine name where the filesystem belongs to
         :param source_path: the absolute source path
@@ -723,6 +520,7 @@ class Firecrest:
             data={"targetPath": target_path, "sourcePath": source_path},
         )
         self._json_response([resp], 201)
+        return target_path
 
     def file_type(self, machine: str, target_path: str) -> str:
         """Uses the `file` linux application to determine the type of a file.
@@ -760,8 +558,9 @@ class Firecrest:
         )
         return self._json_response([resp], 200)["output"]
 
-    def symlink(self, machine: str, target_path: str, link_path: str) -> None:
+    def symlink(self, machine: str, target_path: str, link_path: str) -> str:
         """Creates a symbolic link.
+        When successful, the method returns a string with the path of the newly created link.
 
         :param machine: the machine name where the filesystem belongs to
         :param target_path: the absolute path that the symlink will point to
@@ -774,6 +573,7 @@ class Firecrest:
             data={"targetPath": target_path, "linkPath": link_path},
         )
         self._json_response([resp], 201)
+        return target_path
 
     def simple_download(
         self, machine: str, source_path: str, target_path: str | pathlib.Path | BytesIO
@@ -868,8 +668,9 @@ class Firecrest:
         self,
         machine: str,
         target_path: str,
-        bytes: Optional[int] = None,
-        lines: Optional[int] = None,
+        bytes: Optional[str] = None,
+        lines: Optional[str] = None,
+        skip_ending: Optional[bool] = False,
     ) -> str:
         """Display the beginning of a specified file.
         By default 10 lines will be returned.
@@ -879,14 +680,20 @@ class Firecrest:
 
         :param machine: the machine name where the filesystem belongs to
         :param target_path: the absolute target path
-        :param lines: the number of lines to be displayed
         :param bytes: the number of bytes to be displayed
+        :param lines: the number of lines to be displayed
+        :param skip_ending: the output will be the whole file, without the last NUM bytes/lines of each file. NUM should be specified in the respective argument through `bytes` or `lines`. Equivalent to passing -NUM to the `head` command.
         :calls: GET `/utilities/head`
         """
         resp = self._get_request(
             endpoint="/utilities/head",
             additional_headers={"X-Machine-Name": machine},
-            params={"targetPath": target_path, "lines": lines, "bytes": bytes},
+            params={
+                "targetPath": target_path,
+                "lines": lines,
+                "bytes": bytes,
+                "skip_ending": skip_ending,
+            },
         )
         return self._json_response([resp], 200)["output"]
 
@@ -894,8 +701,9 @@ class Firecrest:
         self,
         machine: str,
         target_path: str,
-        bytes: Optional[int] = None,
-        lines: Optional[int] = None,
+        bytes: Optional[str] = None,
+        lines: Optional[str] = None,
+        skip_beginning: Optional[bool] = False,
     ) -> str:
         """Display the last part of a specified file.
         By default 10 lines will be returned.
@@ -905,14 +713,20 @@ class Firecrest:
 
         :param machine: the machine name where the filesystem belongs to
         :param target_path: the absolute target path
-        :param lines: the number of lines to be displayed
         :param bytes: the number of bytes to be displayed
+        :param lines: the number of lines to be displayed
+        :param skip_beginning: the output will start with byte/line NUM of each file. NUM should be specified in the respective argument through `bytes` or `lines`. Equivalent to passing +NUM to the `tail` command.
         :calls: GET `/utilities/head`
         """
         resp = self._get_request(
             endpoint="/utilities/tail",
             additional_headers={"X-Machine-Name": machine},
-            params={"targetPath": target_path, "lines": lines, "bytes": bytes},
+            params={
+                "targetPath": target_path,
+                "lines": lines,
+                "bytes": bytes,
+                "skip_beginning": skip_beginning,
+            },
         )
         return self._json_response([resp], 200)["output"]
 
@@ -932,13 +746,20 @@ class Firecrest:
         )
         return self._json_response([resp], 200)["output"]
 
-    def whoami(self) -> Optional[str]:
+    def whoami(self, machine=None) -> Optional[str]:
         """Returns the username that FirecREST will be using to perform the other calls.
-        Will return `None` if the token is not valid.
+        In the case the machine name is passed in the arguments, a call is made to the respective endpoint and the command whoami is run on the machine.
+        Otherwise, the library decodes the token and will return `None` if the token is not valid.
+
+        :calls: GET `/utilities/whoami`
         """
+        if machine:
+            resp = self._get_request(
+                endpoint="/utilities/whoami",
+                additional_headers={"X-Machine-Name": machine},
+            )
+            return self._json_response([resp], 200)["output"]
 
-        # FIXME This needs to be added as an endpoint in FirecREST,
-        # now it's making a guess and it could be wrong.
         try:
             decoded = jwt.decode(
                 self._authorization.get_access_token(),
@@ -959,14 +780,16 @@ class Firecrest:
             return None
 
     # Compute
-    def _submit_request(self, machine: str, job_script, local_file, account=None):
+    def _submit_request(self, machine: str, job_script, local_file, account=None, env_vars=None):
+        data = {}
+        if account:
+            data["account"] = account
+
+        if env_vars:
+            data["env"] = env_vars
+
         if local_file:
             with open(job_script, "rb") as f:
-                if account:
-                    data = {"account": account}
-                else:
-                    data = None
-
                 resp = self._post_request(
                     endpoint="/compute/jobs/upload",
                     additional_headers={"X-Machine-Name": machine},
@@ -974,10 +797,7 @@ class Firecrest:
                     data=data,
                 )
         else:
-            data = {"targetPath": job_script}
-            if account:
-                data["account"] = account
-
+            data["targetPath"] = job_script
             resp = self._post_request(
                 endpoint="/compute/jobs/path",
                 additional_headers={"X-Machine-Name": machine},
@@ -987,11 +807,17 @@ class Firecrest:
         self._current_method_requests.append(resp)
         return self._json_response(self._current_method_requests, 201)
 
-    def _squeue_request(self, machine: str, jobs=None):
+    def _squeue_request(self, machine: str, jobs=None, page_size=None, page_number=None):
         jobs = [] if jobs is None else jobs
         params = {}
         if jobs:
-            params = {"jobs": ",".join([str(j) for j in jobs])}
+            params["jobs"] = ",".join([str(j) for j in jobs])
+
+        if page_size is not None:
+            params["pageSize"] = page_size
+
+        if page_number is not None:
+            params["pageNumber"] = page_number
 
         resp = self._get_request(
             endpoint="/compute/jobs",
@@ -1001,7 +827,7 @@ class Firecrest:
         self._current_method_requests.append(resp)
         return self._json_response(self._current_method_requests, 200)
 
-    def _acct_request(self, machine: str, jobs=None, start_time=None, end_time=None):
+    def _acct_request(self, machine: str, jobs=None, start_time=None, end_time=None, page_size=None, page_number=None):
         jobs = [] if jobs is None else jobs
         params = {}
         if jobs:
@@ -1013,6 +839,12 @@ class Firecrest:
         if end_time:
             params["endtime"] = end_time
 
+        if page_size is not None:
+            params["pageSize"] = page_size
+
+        if page_number is not None:
+            params["pageNumber"] = page_number
+
         resp = self._get_request(
             endpoint="/compute/acct",
             additional_headers={"X-Machine-Name": machine},
@@ -1024,26 +856,92 @@ class Firecrest:
     def submit(
         self,
         machine: str,
-        job_script: str,
+        job_script: Optional[str] = None,
         local_file: Optional[bool] = True,
+        script_str: Optional[str] = None,
+        script_local_path: Optional[str] = None,
+        script_remote_path: Optional[str] = None,
         account: Optional[str] = None,
+        env_vars: Optional[dict[str, Any]] = None,
     ) -> t.JobSubmit:
-        """Submits a batch script to SLURM on the target system
+        """Submits a batch script to SLURM on the target system. One of `script_str`, `script_local` and `script_remote` needs to be set.
 
         :param machine: the machine name where the scheduler belongs to
-        :param job_script: the path of the script (if it's local it can be relative path, if it is on the machine it has to be the absolute path)
-        :param local_file: batch file can be local (default) or on the machine's filesystem
+        :param job_script: [deprecated] use `script_str`, `script_local_path` or `script_remote_path`
+        :param local_file: [deprecated]
+        :param script_str: the content of the script to be submitted
+        :param script_local_path: the path of the script on the local file system
+        :param script_remote_path: the full path of the script on the remote file system
         :param account: submit the job with this project account
+        :param env_vars: dictionary (varName, value) defining environment variables to be exported for the job
         :calls: POST `/compute/jobs/upload` or POST `/compute/jobs/path`
 
                 GET `/tasks/{taskid}`
         """
+        if [
+            script_str is None,
+            script_local_path is None,
+            script_remote_path is None,
+            job_script is None
+        ].count(False) != 1:
+            logger.error(
+                "Only one of the arguments  `script_str`, `script_local_path`, "
+                "`script_remote_path`, and `job_script` can be set at a time. "
+                "`job_script` is deprecated, so prefer one of the others."
+            )
+            raise ValueError(
+                "Only one of the arguments  `script_str`, `script_local_path`, "
+                "`script_remote_path`, and `job_script` can be set at a time. "
+            )
+
+        if job_script is not None:
+            logger.warning("`local_file` argument is deprecated, please use one of "
+                           "`script_str`, `script_local_path` or `script_remote_path` instead")
+
+            if local_file:
+                script_local_path = job_script
+            else:
+                script_remote_path = job_script
+
+        if script_str is not None:
+            is_path = False
+            is_local = True
+            job_script_file = None
+        elif script_local_path is not None:
+            is_path = True
+            is_local = True
+            job_script_file = script_local_path
+        elif script_remote_path is not None:
+            is_path = True
+            is_local = False
+            job_script_file = script_remote_path
+
         self._current_method_requests = []
-        json_response = self._submit_request(machine, job_script, local_file, account)
-        logger.info(f"Job submission task: {json_response['task_id']}")
-        return self._poll_tasks(
+
+        # Check if `job_script` is a filename or a job script and create a file if necessary
+        context: Any = (
+            tempfile.TemporaryDirectory()
+            if not is_path
+            else nullcontext(None)
+        )
+        with context as tmpdirname:
+            if not is_path:
+                logger.info(f"Created temporary directory {tmpdirname}")
+                with open(os.path.join(tmpdirname, "script.batch"), "w") as temp_file:
+                    temp_file.write(script_str)  # type: ignore
+
+                job_script_file = os.path.join(tmpdirname, "script.batch")
+
+            env = json.dumps(env_vars) if env_vars else None
+            json_response = self._submit_request(machine, job_script_file, is_local, account, env)
+            logger.info(f"Job submission task: {json_response['task_id']}")
+
+        # Inject taskid in the result
+        result = self._poll_tasks(
             json_response["task_id"], "200", itertools.cycle([1, 5, 10])
         )
+        result["firecrest_taskid"] = json_response["task_id"]
+        return result
 
     def poll(
         self,
@@ -1051,6 +949,8 @@ class Firecrest:
         jobs: Optional[Sequence[str | int]] = None,
         start_time: Optional[str] = None,
         end_time: Optional[str] = None,
+        page_size: Optional[int] = None,
+        page_number: Optional[int] = None,
     ) -> List[t.JobAcct]:
         """Retrieves information about submitted jobs.
         This call uses the `sacct` command.
@@ -1059,13 +959,20 @@ class Firecrest:
         :param jobs: list of the IDs of the jobs
         :param start_time: Start time (and/or date) of job's query. Allowed formats are HH:MM[:SS] [AM|PM] MMDD[YY] or MM/DD[/YY] or MM.DD[.YY] MM/DD[/YY]-HH:MM[:SS] YYYY-MM-DD[THH:MM[:SS]]
         :param end_time: End time (and/or date) of job's query. Allowed formats are HH:MM[:SS] [AM|PM] MMDD[YY] or MM/DD[/YY] or MM.DD[.YY] MM/DD[/YY]-HH:MM[:SS] YYYY-MM-DD[THH:MM[:SS]]
+        :param page_size: number of entries returned (when `page_number` is not `None`, the default value is 25)
+        :param page_number: page number (if set to `None` the default value is 0)
         :calls: GET `/compute/acct`
 
                 GET `/tasks/{taskid}`
         """
         self._current_method_requests = []
+        if isinstance(jobs, str):
+            logger.warning(
+                f"`jobs` is meant to be a list, not a string. Will poll for jobs: {[str(j) for j in jobs]}"
+            )
+
         jobids = [str(j) for j in jobs] if jobs else []
-        json_response = self._acct_request(machine, jobids, start_time, end_time)
+        json_response = self._acct_request(machine, jobids, start_time, end_time, page_size, page_number)
         logger.info(f"Job polling task: {json_response['task_id']}")
         res = self._poll_tasks(
             json_response["task_id"], "200", itertools.cycle([1, 5, 10])
@@ -1077,21 +984,32 @@ class Firecrest:
             return res
 
     def poll_active(
-        self, machine: str, jobs: Optional[Sequence[str | int]] = None
+        self,
+        machine: str,
+        jobs: Optional[Sequence[str | int]] = None,
+        page_size: Optional[int] = None,
+        page_number: Optional[int] = None,
     ) -> List[t.JobQueue]:
         """Retrieves information about active jobs.
         This call uses the `squeue -u <username>` command.
 
         :param machine: the machine name where the scheduler belongs to
         :param jobs: list of the IDs of the jobs
+        :param page_size: number of entries returned (when `page_number` is not `None`, the default value is 25)
+        :param page_number: page number (if set to `None` the default value is 0)
         :calls: GET `/compute/jobs`
 
                 GET `/tasks/{taskid}`
         """
         self._current_method_requests = []
+        if isinstance(jobs, str):
+            logger.warning(
+                f"`jobs` is meant to be a list, not a string. Will poll for jobs: {[str(j) for j in jobs]}"
+            )
+
         jobs = jobs if jobs else []
         jobids = [str(j) for j in jobs]
-        json_response = self._squeue_request(machine, jobids)
+        json_response = self._squeue_request(machine, jobids, page_size, page_number)
         logger.info(f"Job active polling task: {json_response['task_id']}")
         dict_result = self._poll_tasks(
             json_response["task_id"], "200", itertools.cycle([1, 5, 10])
@@ -1137,7 +1055,7 @@ class Firecrest:
             data["sourcePath"] = source_path
 
         if job_name:
-            data["jobname"] = job_name
+            data["jobName"] = job_name
 
         if time:
             data["time"] = time
