@@ -5,7 +5,9 @@
 #  SPDX-License-Identifier: BSD-3-Clause
 #
 import logging
+import re
 import typer
+import yaml
 
 import firecrest as fc
 
@@ -82,7 +84,7 @@ def examine_exeption(e: Exception) -> None:
 
 def create_table(table_title, data, *mappings):
     table = Table(title=table_title, box=box.ASCII)
-    for (title, _) in mappings:
+    for title, _ in mappings:
         table.add_column(title, overflow="fold")
 
     for i in data:
@@ -97,14 +99,31 @@ def version_callback(value: bool):
         raise typer.Exit()
 
 
+def config_parent_load_callback(ctx: typer.Context, param: typer.CallbackParam, value: str):
+    ctx.default_map = ctx.parent.default_map  # type: ignore
+
+
+def config_callback(ctx: typer.Context, param: typer.CallbackParam, value: str):
+    if value:
+        try:
+            with open(value, 'r') as f:
+                config = yaml.safe_load(f)
+
+            ctx.default_map = ctx.default_map or {}
+            ctx.default_map.update(config)
+        except Exception as ex:
+            raise typer.BadParameter(str(ex))
+
+    return value
+
+
 @app.command(rich_help_panel="Status commands")
 def services(
     name: Optional[str] = typer.Option(
         None, "-n", "--name", help="Get information for only one service."
     )
 ):
-    """Provides information for the services of FirecREST
-    """
+    """Provides information for the services of FirecREST"""
     try:
         if name:
             result = [client.service(name)]
@@ -132,8 +151,7 @@ def systems(
         None, "-n", "--name", help="Get information for only one system."
     )
 ):
-    """Provides information for the available systems in FirecREST
-    """
+    """Provides information for the available systems in FirecREST"""
     try:
         if name:
             result = [client.system(name)]
@@ -157,24 +175,30 @@ def systems(
 
 @app.command(rich_help_panel="Status commands")
 def parameters():
-    """Configurable parameters of FirecREST
-    """
+    """Configurable parameters of FirecREST"""
     try:
         result = client.parameters()
-        storage_table = create_table(
-            "Storage parameters",
-            result["storage"],
+        info = [
             ("Name", "name"),
             ("Value", "value"),
             ("Unit", "unit"),
+        ]
+        if "description" in result["storage"][0]:
+            info.append(("Description", "description"))
+
+        storage_table = create_table(
+            "Storage parameters",
+            result["storage"],
+            *info
         )
 
+        if "description" in result["utilities"][0]:
+            info.append(("Description", "description"))
+
         utilities_table = create_table(
             "Utilities parameters",
             result["utilities"],
-            ("Name", "name"),
-            ("Value", "value"),
-            ("Unit", "unit"),
+            *info
         )
 
         console.print(storage_table, utilities_table, overflow="fold")
@@ -192,8 +216,7 @@ def tasks(
         True, help="Display the output in a pager application."
     ),
 ):
-    """Retrieve information about the FirecREST tasks of the users
-    """
+    """Retrieve information about the FirecREST tasks of the users"""
     try:
         result = client._tasks(taskids)
         num_results = len(result.values())
@@ -222,8 +245,17 @@ def tasks(
 
 @app.command(rich_help_panel="Utilities commands")
 def ls(
-    machine: str = typer.Argument(
-        ..., help="The machine name where the filesystem belongs to."
+    config_from_parent: str = typer.Option(None,
+        callback=config_parent_load_callback,
+        is_eager=True,
+        hidden=True
+    ),
+    system: str = typer.Option(
+        ...,
+        "-s",
+        "--system",
+        help="The name of the system where the filesystem belongs to.",
+        envvar="FIRECREST_SYSTEM",
     ),
     path: str = typer.Argument(..., help="The absolute target path."),
     show_hidden: bool = typer.Option(
@@ -234,15 +266,14 @@ def ls(
     ),
     raw: bool = typer.Option(False, "--raw", help="Print unformatted."),
 ):
-    """List directory contents
-    """
+    """List directory contents"""
     try:
-        result = client.list_files(machine, path, show_hidden)
+        result = client.list_files(system, path, show_hidden)
         if raw:
             console.print(result)
         else:
             table = create_table(
-                f"Files in machine `{machine}` and path `{path}`",
+                f"Files in system `{system}` and path `{path}`",
                 result,
                 ("Filename", "name"),
                 ("Type", "type"),
@@ -262,8 +293,17 @@ def ls(
 
 @app.command(rich_help_panel="Utilities commands")
 def mkdir(
-    machine: str = typer.Argument(
-        ..., help="The machine name where the filesystem belongs to."
+    config_from_parent: str = typer.Option(None,
+        callback=config_parent_load_callback,
+        is_eager=True,
+        hidden=True
+    ),
+    system: str = typer.Option(
+        ...,
+        "-s",
+        "--system",
+        help="The name of the system where the filesystem belongs to.",
+        envvar="FIRECREST_SYSTEM",
     ),
     path: str = typer.Argument(..., help="The absolute target path."),
     p: bool = typer.Option(
@@ -272,10 +312,9 @@ def mkdir(
         help="Create intermediate directories as required, equivalent to `-p` of the unix command.",
     ),
 ):
-    """Create new directories
-    """
+    """Create new directories"""
     try:
-        client.mkdir(machine, path, p)
+        client.mkdir(system, path, p)
     except Exception as e:
         examine_exeption(e)
         raise typer.Exit(code=1)
@@ -283,16 +322,24 @@ def mkdir(
 
 @app.command(rich_help_panel="Utilities commands")
 def mv(
-    machine: str = typer.Argument(
-        ..., help="The machine name where the filesystem belongs to."
+    config_from_parent: str = typer.Option(None,
+        callback=config_parent_load_callback,
+        is_eager=True,
+        hidden=True
+    ),
+    system: str = typer.Option(
+        ...,
+        "-s",
+        "--system",
+        help="The name of the system where the filesystem belongs to.",
+        envvar="FIRECREST_SYSTEM",
     ),
     source: str = typer.Argument(..., help="The absolute source path."),
     destination: str = typer.Argument(..., help="The absolute destination path."),
 ):
-    """Rename/move files, directory, or symlink at the `source_path` to the `target_path` on `machine`'s filesystem
-    """
+    """Rename/move files, directory, or symlink at the `source_path` to the `target_path` on `system`'s filesystem"""
     try:
-        client.mv(machine, source, destination)
+        client.mv(system, source, destination)
     except Exception as e:
         examine_exeption(e)
         raise typer.Exit(code=1)
@@ -300,16 +347,24 @@ def mv(
 
 @app.command(rich_help_panel="Utilities commands")
 def chmod(
-    machine: str = typer.Argument(
-        ..., help="The machine name where the filesystem belongs to."
+    config_from_parent: str = typer.Option(None,
+        callback=config_parent_load_callback,
+        is_eager=True,
+        hidden=True
+    ),
+    system: str = typer.Option(
+        ...,
+        "-s",
+        "--system",
+        help="The name of the system where the filesystem belongs to.",
+        envvar="FIRECREST_SYSTEM",
     ),
     path: str = typer.Argument(..., help="The absolute target path."),
     mode: str = typer.Argument(..., help="Same as numeric mode of linux chmod tool."),
 ):
-    """Change the file mod bits of a given file according to the specified mode
-    """
+    """Change the file mod bits of a given file according to the specified mode"""
     try:
-        client.chmod(machine, path, mode)
+        client.chmod(system, path, mode)
     except Exception as e:
         examine_exeption(e)
         raise typer.Exit(code=1)
@@ -317,8 +372,17 @@ def chmod(
 
 @app.command(rich_help_panel="Utilities commands")
 def chown(
-    machine: str = typer.Argument(
-        ..., help="The machine name where the filesystem belongs to."
+    config_from_parent: str = typer.Option(None,
+        callback=config_parent_load_callback,
+        is_eager=True,
+        hidden=True
+    ),
+    system: str = typer.Option(
+        ...,
+        "-s",
+        "--system",
+        help="The name of the system where the filesystem belongs to.",
+        envvar="FIRECREST_SYSTEM",
     ),
     path: str = typer.Argument(..., help="The absolute target path."),
     owner: Optional[str] = typer.Option(None, help="Owner ID for target."),
@@ -329,7 +393,7 @@ def chown(
     If only owner or group information is passed, only that information will be updated.
     """
     try:
-        client.chown(machine, path, owner, group)
+        client.chown(system, path, owner, group)
     except Exception as e:
         examine_exeption(e)
         raise typer.Exit(code=1)
@@ -337,16 +401,24 @@ def chown(
 
 @app.command(rich_help_panel="Utilities commands")
 def cp(
-    machine: str = typer.Argument(
-        ..., help="The machine name where the filesystem belongs to."
+    config_from_parent: str = typer.Option(None,
+        callback=config_parent_load_callback,
+        is_eager=True,
+        hidden=True
+    ),
+    system: str = typer.Option(
+        ...,
+        "-s",
+        "--system",
+        help="The name of the system where the filesystem belongs to.",
+        envvar="FIRECREST_SYSTEM",
     ),
     source: str = typer.Argument(..., help="The absolute source path."),
     destination: str = typer.Argument(..., help="The absolute destination path."),
 ):
-    """Copy files
-    """
+    """Copy files"""
     try:
-        client.copy(machine, source, destination)
+        client.copy(system, source, destination)
     except Exception as e:
         examine_exeption(e)
         raise typer.Exit(code=1)
@@ -354,15 +426,23 @@ def cp(
 
 @app.command(rich_help_panel="Utilities commands")
 def file(
-    machine: str = typer.Argument(
-        ..., help="The machine name where the filesystem belongs to."
+    config_from_parent: str = typer.Option(None,
+        callback=config_parent_load_callback,
+        is_eager=True,
+        hidden=True
+    ),
+    system: str = typer.Option(
+        ...,
+        "-s",
+        "--system",
+        help="The name of the system where the filesystem belongs to.",
+        envvar="FIRECREST_SYSTEM",
     ),
     path: str = typer.Argument(..., help="The absolute target path."),
 ):
-    """Determine file type
-    """
+    """Determine file type"""
     try:
-        console.print(client.file_type(machine, path))
+        console.print(client.file_type(system, path))
     except Exception as e:
         examine_exeption(e)
         raise typer.Exit(code=1)
@@ -370,17 +450,25 @@ def file(
 
 @app.command(rich_help_panel="Utilities commands")
 def stat(
-    machine: str = typer.Argument(
-        ..., help="The machine name where the filesystem belongs to."
+    config_from_parent: str = typer.Option(None,
+        callback=config_parent_load_callback,
+        is_eager=True,
+        hidden=True
+    ),
+    system: str = typer.Option(
+        ...,
+        "-s",
+        "--system",
+        help="The name of the system where the filesystem belongs to.",
+        envvar="FIRECREST_SYSTEM",
     ),
     path: str = typer.Argument(..., help="The absolute target path."),
     deref: bool = typer.Option(False, "-L", "--dereference", help="Follow links."),
     raw: bool = typer.Option(False, "--raw", help="Print unformatted."),
 ):
-    """Use the stat linux application to determine the status of a file on the machine's filesystem
-    """
+    """Use the stat linux application to determine the status of a file on the system's filesystem"""
     try:
-        result = client.stat(machine, path, deref)
+        result = client.stat(system, path, deref)
         if raw:
             console.print(result)
         else:
@@ -456,16 +544,24 @@ def stat(
 
 @app.command(rich_help_panel="Utilities commands")
 def symlink(
-    machine: str = typer.Argument(
-        ..., help="The machine name where the filesystem belongs to."
+    config_from_parent: str = typer.Option(None,
+        callback=config_parent_load_callback,
+        is_eager=True,
+        hidden=True
+    ),
+    system: str = typer.Option(
+        ...,
+        "-s",
+        "--system",
+        help="The name of the system where the filesystem belongs to.",
+        envvar="FIRECREST_SYSTEM",
     ),
     target: str = typer.Argument(..., help="The path of the original file."),
     link_name: str = typer.Argument(..., help="The name of the link to the TARGET."),
 ):
-    """Create a symbolic link
-    """
+    """Create a symbolic link"""
     try:
-        client.symlink(machine, target, link_name)
+        client.symlink(system, target, link_name)
     except Exception as e:
         examine_exeption(e)
         raise typer.Exit(code=1)
@@ -473,8 +569,19 @@ def symlink(
 
 @app.command(rich_help_panel="Utilities commands")
 def rm(
-    machine: str,
-    path: str,
+    config_from_parent: str = typer.Option(None,
+        callback=config_parent_load_callback,
+        is_eager=True,
+        hidden=True
+    ),
+    system: str = typer.Option(
+        ...,
+        "-s",
+        "--system",
+        help="The name of the system where the filesystem belongs to.",
+        envvar="FIRECREST_SYSTEM",
+    ),
+    path: str = typer.Argument(..., help="The absolute target path."),
     force: bool = typer.Option(
         ...,
         prompt="Are you sure you want to delete this entry?",
@@ -482,11 +589,10 @@ def rm(
     ),
     # TODO (?) add option to not display error to emulate `-f` from the rm command
 ):
-    """Remove directory entries
-    """
+    """Remove directory entries"""
     try:
         if force:
-            client.simple_delete(machine, path)
+            client.simple_delete(system, path)
         else:
             console.print("Operation cancelled")
     except Exception as e:
@@ -496,15 +602,23 @@ def rm(
 
 @app.command(rich_help_panel="Utilities commands")
 def checksum(
-    machine: str = typer.Argument(
-        ..., help="The machine name where the filesystem belongs to."
+    config_from_parent: str = typer.Option(None,
+        callback=config_parent_load_callback,
+        is_eager=True,
+        hidden=True
+    ),
+    system: str = typer.Option(
+        ...,
+        "-s",
+        "--system",
+        help="The name of the system where the filesystem belongs to.",
+        envvar="FIRECREST_SYSTEM",
     ),
     path: str = typer.Argument(..., help="The absolute target path."),
 ):
-    """Calculate the SHA256 (256-bit) checksum
-    """
+    """Calculate the SHA256 (256-bit) checksum"""
     try:
-        console.print(client.checksum(machine, path))
+        console.print(client.checksum(system, path))
     except Exception as e:
         examine_exeption(e)
         raise typer.Exit(code=1)
@@ -512,15 +626,32 @@ def checksum(
 
 @app.command(rich_help_panel="Utilities commands")
 def head(
-    machine: str = typer.Argument(
-        ..., help="The machine name where the filesystem belongs to."
+    config_from_parent: str = typer.Option(None,
+        callback=config_parent_load_callback,
+        is_eager=True,
+        hidden=True
+    ),
+    system: str = typer.Option(
+        ...,
+        "-s",
+        "--system",
+        help="The name of the system where the filesystem belongs to.",
+        envvar="FIRECREST_SYSTEM",
     ),
     path: str = typer.Argument(..., help="The absolute target path."),
-    lines: int = typer.Option(
-        None, "-n", "--lines", help="Print count lines of each of the specified files."
+    lines: str = typer.Option(
+        None,
+        "-n",
+        "--lines",
+        help="Print NUM lines of each of the specified files; with a leading '-', print all but the last NUM lines of each file",
+        metavar="[-]NUM",
     ),
-    bytes: int = typer.Option(
-        None, "-c", "--bytes", help="Print bytes of each of the specified files."
+    bytes: str = typer.Option(
+        None,
+        "-c",
+        "--bytes",
+        help="Print NUM bytes of each of the specified files; with a leading '-', print all but the last NUM bytes of each file",
+        metavar="[-]NUM",
     ),
 ):
     """Display the beginning of a specified file.
@@ -537,7 +668,17 @@ def head(
         raise typer.Exit(code=1)
 
     try:
-        console.print(client.head(machine, path, bytes, lines))
+        lines_arg = lines
+        bytes_arg = bytes
+        skip_ending = False
+        if lines and lines.startswith("-"):
+            lines_arg = lines[1:]
+            skip_ending = True
+        elif bytes and bytes.startswith("-"):
+            bytes_arg = bytes[1:]
+            skip_ending = True
+
+        console.print(client.head(system, path, bytes_arg, lines_arg, skip_ending))
     except Exception as e:
         examine_exeption(e)
         raise typer.Exit(code=1)
@@ -545,15 +686,32 @@ def head(
 
 @app.command(rich_help_panel="Utilities commands")
 def tail(
-    machine: str = typer.Argument(
-        ..., help="The machine name where the filesystem belongs to."
+    config_from_parent: str = typer.Option(None,
+        callback=config_parent_load_callback,
+        is_eager=True,
+        hidden=True
+    ),
+    system: str = typer.Option(
+        ...,
+        "-s",
+        "--system",
+        help="The name of the system where the filesystem belongs to.",
+        envvar="FIRECREST_SYSTEM",
     ),
     path: str = typer.Argument(..., help="The absolute target path."),
-    lines: int = typer.Option(
-        None, "-n", "--lines", help="Print count lines of each of the specified files."
+    lines: str = typer.Option(
+        None,
+        "-n",
+        "--lines",
+        help="Output the last NUM lines; or use +NUM to output starting with line NUM",
+        metavar="[+]NUM",
     ),
-    bytes: int = typer.Option(
-        None, "-c", "--bytes", help="Print bytes of each of the specified files."
+    bytes: str = typer.Option(
+        None,
+        "-c",
+        "--bytes",
+        help="Output the last NUM bytes; or use +NUM to output starting with byte NUM",
+        metavar="[+]NUM",
     ),
 ):
     """Display the end of a specified file.
@@ -570,18 +728,42 @@ def tail(
         raise typer.Exit(code=1)
 
     try:
-        console.print(client.tail(machine, path, bytes, lines))
+        lines_arg = lines
+        bytes_arg = bytes
+        skip_beginning = False
+        if lines and lines.startswith("+"):
+            lines_arg = lines[1:]
+            skip_beginning = True
+        elif bytes and bytes.startswith("+"):
+            bytes_arg = bytes[1:]
+            skip_beginning = True
+
+        console.print(client.tail(system, path, bytes_arg, lines_arg, skip_beginning))
     except Exception as e:
         examine_exeption(e)
         raise typer.Exit(code=1)
 
 
 @app.command(rich_help_panel="Utilities commands")
-def whoami():
-    """Return the username that FirecREST will be using to perform the other calls
+def whoami(
+    config_from_parent: str = typer.Option(None,
+        callback=config_parent_load_callback,
+        is_eager=True,
+        hidden=True
+    ),
+    system: str = typer.Option(
+        None,
+        "-s",
+        "--system",
+        help="The name of the system where the `whoami` command will run.",
+        envvar="FIRECREST_SYSTEM",
+    ),
+):
+    """Return the username that FirecREST will be using to perform the other calls.
+    If the name of the system is not passed the username will be deduced from the token.
     """
     try:
-        console.print(client.whoami())
+        console.print(client.whoami(system))
     except Exception as e:
         examine_exeption(e)
         raise typer.Exit(code=1)
@@ -594,8 +776,17 @@ class TransferType(str, Enum):
 
 @app.command(rich_help_panel="Storage commands")
 def download(
-    machine: str = typer.Argument(
-        ..., help="The machine name where the source filesystem belongs to."
+    config_from_parent: str = typer.Option(None,
+        callback=config_parent_load_callback,
+        is_eager=True,
+        hidden=True
+    ),
+    system: str = typer.Option(
+        ...,
+        "-s",
+        "--system",
+        help="The name of the system where the source filesystem belongs to.",
+        envvar="FIRECREST_SYSTEM",
     ),
     source: str = typer.Argument(..., help="The absolute source path."),
     destination: Optional[str] = typer.Argument(
@@ -620,12 +811,12 @@ def download(
     try:
         if transfer_type == TransferType.direct:
             if destination:
-                client.simple_download(machine, source, destination)
+                client.simple_download(system, source, destination)
             else:
                 console.print("`destination` is required when the ")
                 raise typer.Exit(code=1)
         elif transfer_type == TransferType.external:
-            down_obj = client.external_download(machine, source)
+            down_obj = client.external_download(system, source)
             console.print(
                 f"Follow the status of the transfer asynchronously with that task ID: [green]{down_obj.task_id}[/green]"
             )
@@ -633,7 +824,7 @@ def download(
                 "Moving file to the staging area... It is safe to "
                 "cancel the command and follow up through the task."
             ):
-                console.out(f"Download the file from:\n{down_obj.object_storage_data}")
+                console.out(f"Download the file from:\n{down_obj.object_storage_link}")
     except Exception as e:
         examine_exeption(e)
         raise typer.Exit(code=1)
@@ -641,8 +832,17 @@ def download(
 
 @app.command(rich_help_panel="Storage commands")
 def upload(
-    machine: str = typer.Argument(
-        ..., help="The machine name where the source filesystem belongs to."
+    config_from_parent: str = typer.Option(None,
+        callback=config_parent_load_callback,
+        is_eager=True,
+        hidden=True
+    ),
+    system: str = typer.Option(
+        ...,
+        "-s",
+        "--system",
+        help="The name of the system where the source filesystem belongs to.",
+        envvar="FIRECREST_SYSTEM",
     ),
     source: str = typer.Argument(..., help="The source path (can be relative)."),
     destination_directory: str = typer.Argument(
@@ -650,7 +850,7 @@ def upload(
     ),
     filename: Optional[str] = typer.Argument(
         None,
-        help="The name of the file in the machine (by default it will be same as the local file). It works only for a direct upload.",
+        help="The name of the file in the system (by default it will be same as the local file). It works only for a direct upload.",
     ),
     transfer_type: TransferType = typer.Option(
         TransferType.direct,
@@ -669,9 +869,9 @@ def upload(
     """
     try:
         if transfer_type == TransferType.direct:
-            client.simple_upload(machine, source, destination_directory, filename)
+            client.simple_upload(system, source, destination_directory, filename)
         elif transfer_type == TransferType.external:
-            up_obj = client.external_upload(machine, source, destination_directory)
+            up_obj = client.external_upload(system, source, destination_directory)
             console.print(
                 f"Follow the status of the transfer asynchronously with that task ID: [green]{up_obj.task_id}[/green]"
             )
@@ -697,14 +897,28 @@ def upload(
         raise typer.Exit(code=1)
 
 
+def validate_env_var_format(value: List[str]):
+    pattern = re.compile(r'\S+=\S+')
+    for item in value:
+        if not pattern.match(item):
+            raise typer.BadParameter(f"Please use the format `VAR=VALUE`.")
+
+    return value
+
+
 @app.command(rich_help_panel="Compute commands")
 def submit(
-    machine: str = typer.Argument(
-        ..., help="The machine name where the source filesystem belongs to."
+    config_from_parent: str = typer.Option(None,
+        callback=config_parent_load_callback,
+        is_eager=True,
+        hidden=True
+    ),
+    system: str = typer.Option(
+        ..., "-s", "--system", help="The name of the system.", envvar="FIRECREST_SYSTEM"
     ),
     job_script: str = typer.Argument(
         ...,
-        help="The path of the script (if it's local it can be relative path, if it is on the machine it has to be the absolute path)",
+        help="The path of the script (if it's local it can be relative path, if it is on the system it has to be the absolute path)",
     ),
     account: Optional[str] = typer.Option(
         None,
@@ -713,13 +927,26 @@ def submit(
     ),
     local: Optional[bool] = typer.Option(
         True,
-        help="The batch file can be local (default) or on the machine's filesystem.",
+        help="The batch file can be local (default) or on the system's filesystem.",
     ),
+    env_vars: List[str] = typer.Option(
+        [], "-e", "--env-var",
+        metavar="VAR=VALUE",
+        help="Environment variable to be exported in the environment where the job script will be submitted (format `VAR=VALUE`).",
+        callback=validate_env_var_format
+    )
 ):
-    """Submit a batch script to the workload manger of the target system
-    """
+    """Submit a batch script to the workload manager of the target system"""
+    envvars = {}
+    for var_value_pair in env_vars:
+        var, val = var_value_pair.split('=', 1)
+        envvars[var] = val
+
     try:
-        console.print(client.submit(machine, job_script, local, account=account))
+        if local:
+            console.print(client.submit(system, script_local_path=job_script, account=account, env_vars=envvars))
+        else:
+            console.print(client.submit(system, script_remote_path=job_script, account=account, env_vars=envvars))
     except Exception as e:
         examine_exeption(e)
         raise typer.Exit(code=1)
@@ -727,8 +954,17 @@ def submit(
 
 @submit_template_app.command("mv")
 def submit_mv(
-    machine: str = typer.Argument(
-        ..., help="The machine name where the source filesystem belongs to."
+    config_from_parent: str = typer.Option(None,
+        callback=config_parent_load_callback,
+        is_eager=True,
+        hidden=True
+    ),
+    system: str = typer.Option(
+        ...,
+        "-s",
+        "--system",
+        help="The name of the system where the filesystem belongs to.",
+        envvar="FIRECREST_SYSTEM",
     ),
     source: str = typer.Argument(..., help="The absolute source path."),
     destination: str = typer.Argument(..., help="The absolute destination path."),
@@ -752,12 +988,11 @@ def submit_mv(
         """,
     ),
 ):
-    """Move/rename file
-    """
+    """Move/rename file"""
     try:
         console.print(
             client.submit_move_job(
-                machine, source, destination, job_name, time, jobid, account
+                system, source, destination, job_name, time, jobid, account
             )
         )
     except Exception as e:
@@ -767,8 +1002,17 @@ def submit_mv(
 
 @submit_template_app.command("cp")
 def submit_cp(
-    machine: str = typer.Argument(
-        ..., help="The machine name where the source filesystem belongs to."
+    config_from_parent: str = typer.Option(None,
+        callback=config_parent_load_callback,
+        is_eager=True,
+        hidden=True
+    ),
+    system: str = typer.Option(
+        ...,
+        "-s",
+        "--system",
+        help="The name of the system where the filesystem belongs to.",
+        envvar="FIRECREST_SYSTEM",
     ),
     source: str = typer.Argument(..., help="The absolute source path."),
     destination: str = typer.Argument(..., help="The absolute destination path."),
@@ -792,12 +1036,11 @@ def submit_cp(
         """,
     ),
 ):
-    """Copy file
-    """
+    """Copy file"""
     try:
         console.print(
             client.submit_copy_job(
-                machine, source, destination, job_name, time, jobid, account
+                system, source, destination, job_name, time, jobid, account
             )
         )
     except Exception as e:
@@ -807,8 +1050,17 @@ def submit_cp(
 
 @submit_template_app.command("rsync")
 def submit_rsync(
-    machine: str = typer.Argument(
-        ..., help="The machine name where the source filesystem belongs to."
+    config_from_parent: str = typer.Option(None,
+        callback=config_parent_load_callback,
+        is_eager=True,
+        hidden=True
+    ),
+    system: str = typer.Option(
+        ...,
+        "-s",
+        "--system",
+        help="The name of the system where the filesystem belongs to.",
+        envvar="FIRECREST_SYSTEM",
     ),
     source: str = typer.Argument(..., help="The absolute source path."),
     destination: str = typer.Argument(..., help="The absolute destination path."),
@@ -832,12 +1084,11 @@ def submit_rsync(
         """,
     ),
 ):
-    """Transfer/synchronize files or directories efficiently between filesystems
-    """
+    """Transfer/synchronize files or directories efficiently between filesystems"""
     try:
         console.print(
             client.submit_rsync_job(
-                machine, source, destination, job_name, time, jobid, account
+                system, source, destination, job_name, time, jobid, account
             )
         )
     except Exception as e:
@@ -847,8 +1098,17 @@ def submit_rsync(
 
 @submit_template_app.command("rm")
 def submit_rm(
-    machine: str = typer.Argument(
-        ..., help="The machine name where the source filesystem belongs to."
+    config_from_parent: str = typer.Option(None,
+        callback=config_parent_load_callback,
+        is_eager=True,
+        hidden=True
+    ),
+    system: str = typer.Option(
+        ...,
+        "-s",
+        "--system",
+        help="The name of the system where the source filesystem belongs to.",
+        envvar="FIRECREST_SYSTEM",
     ),
     path: str = typer.Argument(..., help="The absolute target path."),
     job_name: Optional[str] = typer.Option(None, help="Job name in the script."),
@@ -871,11 +1131,10 @@ def submit_rm(
         """,
     ),
 ):
-    """Remove files
-    """
+    """Remove files"""
     try:
         console.print(
-            client.submit_delete_job(machine, path, job_name, time, jobid, account)
+            client.submit_delete_job(system, path, job_name, time, jobid, account)
         )
     except Exception as e:
         examine_exeption(e)
@@ -884,8 +1143,13 @@ def submit_rm(
 
 @app.command(rich_help_panel="Compute commands")
 def poll(
-    machine: str = typer.Argument(
-        ..., help="The machine name where the source filesystem belongs to."
+    config_from_parent: str = typer.Option(None,
+        callback=config_parent_load_callback,
+        is_eager=True,
+        hidden=True
+    ),
+    system: str = typer.Option(
+        ..., "-s", "--system", help="The name of the system.", envvar="FIRECREST_SYSTEM"
     ),
     jobs: Optional[List[str]] = typer.Argument(
         None, help="List of job IDs to display."
@@ -904,7 +1168,7 @@ def poll(
     This call uses the `sacct` command
     """
     try:
-        result = client.poll(machine, jobs, start_time, end_time)
+        result = client.poll(system, jobs, start_time, end_time)
         if raw:
             console.print(result)
         else:
@@ -930,8 +1194,13 @@ def poll(
 
 @app.command(rich_help_panel="Compute commands")
 def poll_active(
-    machine: str = typer.Argument(
-        ..., help="The machine name where the source filesystem belongs to."
+    config_from_parent: str = typer.Option(None,
+        callback=config_parent_load_callback,
+        is_eager=True,
+        hidden=True
+    ),
+    system: str = typer.Option(
+        ..., "-s", "--system", help="The name of the system.", envvar="FIRECREST_SYSTEM"
     ),
     jobs: Optional[List[str]] = typer.Argument(
         None, help="List of job IDs to display."
@@ -942,7 +1211,7 @@ def poll_active(
     This call uses the `squeue -u <username>` command
     """
     try:
-        result = client.poll_active(machine, jobs)
+        result = client.poll_active(system, jobs)
         if raw:
             console.print(result)
         else:
@@ -968,15 +1237,19 @@ def poll_active(
 
 @app.command(rich_help_panel="Compute commands")
 def cancel(
-    machine: str = typer.Argument(
-        ..., help="The machine name where the source filesystem belongs to."
+    config_from_parent: str = typer.Option(None,
+        callback=config_parent_load_callback,
+        is_eager=True,
+        hidden=True
+    ),
+    system: str = typer.Option(
+        ..., "-s", "--system", help="The name of the system.", envvar="FIRECREST_SYSTEM"
     ),
     job: str = typer.Argument(..., help="The ID of the job that will be cancelled."),
 ):
-    """Cancel job
-    """
+    """Cancel job"""
     try:
-        client.cancel(machine, job)
+        client.cancel(system, job)
     except Exception as e:
         examine_exeption(e)
         raise typer.Exit(code=1)
@@ -984,14 +1257,18 @@ def cancel(
 
 @reservation_app.command(rich_help_panel="Reservation commands")
 def list(
-    machine: str = typer.Argument(
-        ..., help="The machine name where the source filesystem belongs to."
-    )
+    config_from_parent: str = typer.Option(None,
+        callback=config_parent_load_callback,
+        is_eager=True,
+        hidden=True
+    ),
+    system: str = typer.Option(
+        ..., "-s", "--system", help="The name of the system.", envvar="FIRECREST_SYSTEM"
+    ),
 ):
-    """List all active reservations and their status
-    """
+    """List all active reservations and their status"""
     try:
-        res = client.all_reservations(machine)
+        res = client.all_reservations(system)
         console.print(res)
     except Exception as e:
         examine_exeption(e)
@@ -1000,8 +1277,13 @@ def list(
 
 @reservation_app.command(rich_help_panel="Reservation commands")
 def create(
-    machine: str = typer.Argument(
-        ..., help="The machine name where the source filesystem belongs to."
+    config_from_parent: str = typer.Option(None,
+        callback=config_parent_load_callback,
+        is_eager=True,
+        hidden=True
+    ),
+    system: str = typer.Option(
+        ..., "-s", "--system", help="The name of the system.", envvar="FIRECREST_SYSTEM"
     ),
     name: str = typer.Argument(..., help="The reservation name."),
     account: str = typer.Argument(
@@ -1018,11 +1300,10 @@ def create(
         ..., help="The end time for reservation (YYYY-MM-DDTHH:MM:SS)."
     ),
 ):
-    """Create a reservation
-    """
+    """Create a reservation"""
     try:
         client.create_reservation(
-            machine, name, account, num_nodes, node_type, start_time, end_time
+            system, name, account, num_nodes, node_type, start_time, end_time
         )
     except Exception as e:
         examine_exeption(e)
@@ -1031,8 +1312,13 @@ def create(
 
 @reservation_app.command(rich_help_panel="Reservation commands")
 def update(
-    machine: str = typer.Argument(
-        ..., help="The machine name where the source filesystem belongs to."
+    config_from_parent: str = typer.Option(None,
+        callback=config_parent_load_callback,
+        is_eager=True,
+        hidden=True
+    ),
+    system: str = typer.Option(
+        ..., "-s", "--system", help="The name of the system.", envvar="FIRECREST_SYSTEM"
     ),
     name: str = typer.Argument(..., help="The reservation name."),
     account: str = typer.Argument(
@@ -1049,11 +1335,10 @@ def update(
         ..., help="The end time for reservation (YYYY-MM-DDTHH:MM:SS)."
     ),
 ):
-    """Update a reservation
-    """
+    """Update a reservation"""
     try:
         client.update_reservation(
-            machine, name, account, num_nodes, node_type, start_time, end_time
+            system, name, account, num_nodes, node_type, start_time, end_time
         )
     except Exception as e:
         examine_exeption(e)
@@ -1062,15 +1347,19 @@ def update(
 
 @reservation_app.command(rich_help_panel="Reservation commands")
 def delete(
-    machine: str = typer.Argument(
-        ..., help="The machine name where the source filesystem belongs to."
+    config_from_parent: str = typer.Option(None,
+        callback=config_parent_load_callback,
+        is_eager=True,
+        hidden=True
+    ),
+    system: str = typer.Option(
+        ..., "-s", "--system", help="The name of the system.", envvar="FIRECREST_SYSTEM"
     ),
     name: str = typer.Argument(..., help="The reservation name."),
 ):
-    """Delete a reservation
-    """
+    """Delete a reservation"""
     try:
-        client.delete_reservation(machine, name)
+        client.delete_reservation(system, name)
     except Exception as e:
         examine_exeption(e)
         raise typer.Exit(code=1)
@@ -1078,6 +1367,12 @@ def delete(
 
 @app.callback()
 def main(
+    config: Optional[str] = typer.Option(
+        None,
+        envvar="FIRECREST_CONFIG",
+        callback=config_callback,
+        is_eager=True,
+    ),
     version: Optional[bool] = typer.Option(
         None,
         "--version",
@@ -1099,6 +1394,12 @@ def main(
         help="URL of the token request in the authorization server (e.g. https://auth.com/auth/.../openid-connect/token).",
         envvar="AUTH_TOKEN_URL",
     ),
+    api_version: str = typer.Option(
+        None,
+        help="Set the version of the api of firecrest. By default it will be assumed that you are using version 1.13.1 or "
+        "compatible. The version is parsed by the `packaging` library.",
+        envvar="FIRECREST_API_VERSION",
+    ),
     verbose: Optional[bool] = typer.Option(
         None, "-v", "--verbose", help="Enable verbose mode."
     ),
@@ -1126,6 +1427,9 @@ def main(
     auth_obj.timeout = auth_timeout
     client = fc.Firecrest(firecrest_url=firecrest_url, authorization=auth_obj)
     client.timeout = timeout
+    if api_version:
+        client.set_api_version(api_version)
+
     if debug:
         logging.basicConfig(
             level=logging.DEBUG,
@@ -1144,3 +1448,6 @@ def main(
             format="%(message)s",
             handlers=[RichHandler(console=console)],
         )
+
+
+typer_click_object = typer.main.get_command(app)
